library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity de2_wm8731_audio is
port (
    clk : in std_logic;       --  Audio CODEC Chip Clock AUD_XCK (18.43 MHz)
    reset_n : in std_logic;
 request : out std_logic;
rom_read     : in std_logic_vector(15 downto 0);
     rom_addr: out std_logic_vector (13 downto 0);

 --   data : in std_logic_vector(15 downto 0);
--	  VGA_LED : out std_logic_vector (26 downto 0);
    read       : in  std_logic;
    write      : in  std_logic;
    chipselect : in  std_logic;
    address    : in  std_logic_vector(5 downto 0);
    readdata   : out std_logic_vector(15 downto 0);
    writedata  : in  std_logic_vector(15 downto 0);
	 
    -- Audio interface signals
 --   AUD_ADCLRCK  : out  std_logic;   --    Audio CODEC ADC LR Clock
    --AUD_ADCDAT   : in   std_logic;   --    Audio CODEC ADC Data
    AUD_DACLRCK  : out  std_logic;   --    Audio CODEC DAC LR Clock
    AUD_DACDAT   : out  std_logic  --    Audio CODEC DAC Data
 --   AUD_BCLK     : inout std_logic  --    Audio CODEC Bit-Stream Clock
  );
end  de2_wm8731_audio;

architecture rtl of de2_wm8731_audio is     

    signal lrck : std_logic;
    signal bclk : std_logic;
  --  signal xck  : std_logic;
    
    signal lrck_divider : unsigned (15 downto 0); 
    signal bclk_divider : unsigned (11 downto 0);
    signal sin_out     : unsigned(15 downto 0);
    signal set_bclk : std_logic;
    signal set_lrck : std_logic;
    signal clr_bclk : std_logic;
    signal lrck_lat : std_logic;
    signal endOfTone : unsigned (13 downto 0);
    signal shift_out : std_logic_vector(15 downto 0);

    signal mark : std_logic;
	 
    signal sin_counter : unsigned (13 downto 0);
	 signal bac_counter : unsigned (4 downto 0); 
    signal shift_before : std_logic_vector(7 downto 0);	 


	type ram_type is array(31 downto 0) of
            std_logic_vector(15 downto 0);
  signal RAM : ram_type:=(
	others=>"0000000000000000"
	);
   signal ram_address: unsigned(4 downto 0);

   signal switch : std_logic ; 
	signal  sin_out0,sin_out1,sin_out2 : unsigned(15 downto 0);
	signal merged: unsigned (15 downto 0);
	
	
	
--  component musicRom is port (Clk: in std_logic;
--addr : in unsigned(12 downto 0);
--data : out unsigned(15 downto 0));
--end component;

--component musicRom1 is port (Clk: in std_logic;
--addr : in unsigned(12 downto 0);
--data : out unsigned(15 downto 0));
--end component;





begin


 ram_address <= unsigned(address(4 downto 0));  
    -- LRCK divider 
    -- Audio chip main clock is 18.432MHz / Sample rate 48KHz
    -- Divider is 18.432 MHz / 48KHz = 192 (X"C0")
    -- Left justify mode set by I2C controller
 
 --storing input signals 
 ----------------------8 bit---------------------------
-- distinguish_audio: process(clk)
-- begin 
--  if rising_edge (clk) then
--    if reset_n ='0' then
--	    mark <='0';
--	 else 
--	    mark <= not mark;
--	 end if;
--       end if;
--	end process distinguish_audio;
  ----------------------8 bit---------------------------
 
 
 
 
  process (clk)
  begin
    if rising_edge(clk) then
      if reset_n = '0' then
        readdata <= (others => '0');       
      else
        if chipselect = '1' then          
            if read = '1' then
              readdata <= RAM( to_integer( ram_address));
            elsif write = '1' then
              RAM(to_integer (ram_address)) <= writedata;
            end if;         
        end if;
      end if;
    end if;
  end process;
  
  
  process (clk)
  begin
    if rising_edge(clk) then
      if reset_n = '0' then 
        lrck_divider <= (others => '0');
      elsif lrck_divider = X"1047"  then        -- "C0" minus 1   822
        lrck_divider <= X"0000";
      else 
        lrck_divider <= lrck_divider + 1;
      end if;
    end if;   
  end process;

  process (clk)
  begin
    if rising_edge(clk) then      
      if reset_n = '0' then 
        bclk_divider <= (others => '0');
      elsif bclk_divider = X"104" or set_lrck = '1'  then ---81 
        bclk_divider <= X"000";
      else 
        bclk_divider <= bclk_divider + 1;
      end if;
    end if;
  end process;

  set_lrck <= '1' when lrck_divider = X"1047" else '0';	--822
    
  process (clk)
  begin
    if rising_edge(clk) then
      if reset_n = '0' then
        lrck <= '0';
      elsif set_lrck = '1' then 
        lrck <= not lrck;
      end if;
    end if;
  end process;
    
  -- BCLK divider
  set_bclk <= '1' when bclk_divider(11 downto 0) = X"082" else '0';--40
  clr_bclk <= '1' when bclk_divider(11 downto 0) = X"104" else '0';--81
  
  process (clk)
  begin
    if rising_edge(clk) then
      if reset_n = '0' then
        bclk <= '0';
      elsif set_lrck = '1' or clr_bclk = '1' then
        bclk <= '0';
      elsif set_bclk = '1' then 
        bclk <= '1';
      end if;
    end if;
  end process;
   ----------------------8 bit---------------------------
--  	audio_out: process (clk)
--	  begin 
--	    if rising_edge(clk) then
--		   if mark= '0' then
--	 shift_before<= sram_data(15 downto 8);
--	   elsif mark ='1' then
--	 shift_before <= sram_data(7 downto 0);
--	 end if;
--  end if;
--end process audio_out;
 ----------------------8 bit---------------------------

  -- Audio data shift output
  process (clk)
  begin
    if rising_edge(clk) then
		if switch = '0'  then
		merged <= unsigned(RAM(to_integer(bac_counter)));
		 ----------------------8 bit---------------------------
--			merged (15 downto 8)<= unsigned (shift_before);
--			merged  (7 downto 0)<= "00000000";
 ----------------------8 bit---------------------------
		else
					--merged <= sin_out;
merged <= unsigned(rom_read);
		end if;
	 
      if reset_n = '0' then
        shift_out <= (others => '0');
      elsif set_lrck = '1' then
        
          shift_out <= std_logic_vector (merged);
    
      elsif clr_bclk = '1' then 
        shift_out <= shift_out (14 downto 0) & '0';
      end if;
    end if;   
  end process;

    -- Audio outputs
    
 --   AUD_ADCLRCK  <= lrck;          
    AUD_DACLRCK  <= lrck;          
    AUD_DACDAT   <= shift_out(15); 
 --   AUD_BCLK     <= bclk;          

    --effect and background sound address counter 
   
	
--	    process(clk)      
--    begin
--      if rising_edge(clk) then
--        if reset_n = '0' or RAM(0) = "0000000000000000"then 
--		 -- if reset_n = '0'  then 
--            sin_counter <= (others => '0');
--        elsif lrck_lat = '1' and lrck = '0'  then  
--          if sin_counter < endOfTone then 
--            
--            sin_counter <= sin_counter + 1;
--				switch <= not switch ;
--				
--		
--          end if;
--        end if;
--      end if;
--    end process;
	 
	 
	    process(clk)      
    begin
      if rising_edge(clk) then
        if reset_n = '0' or RAM(0) = "0000000000000000"then 
		--  if reset_n = '0'  then 
            sin_counter <= (others => '0');
        --dead--
				elsif RAM(0) = "0000000000000001"then 
					sin_counter <= (others => '0');
		  	--hitting--
				elsif RAM(0) = "0000000000000011"then 
					sin_counter <= "01100010101010";
					--laugh--
				elsif RAM(0) = "0000000000000101"then 
					sin_counter <= "10100110011010";
				elsif 
		  	  lrck_lat = '1' and lrck = '0' and sin_counter < endOfTone then 
        
            sin_counter <= sin_counter + 1;
				switch <= not switch ;
				
		
         
        end if;
      end if;
    end process;
	 
	 
--    process(clk)      
--    begin
--      if rising_edge(clk) then
--        if reset_n = '0' or RAM(2) = "0000000000000000"then 
--		 -- if reset_n = '0'  then 
--            sin_counter <= (others => '0');
--        elsif lrck_lat = '1' and lrck = '0'  then  
--          if sin_counter = endOfTone then 
--            sin_counter <= "0000000000000";
--          else  
--            sin_counter <= sin_counter + 1;
--				switch <= not switch ;
--		
--          end if;
--        end if;
--      end if;
--    end process;

--	    process(clk)      
--    begin
--      if rising_edge(clk) then
--        if reset_n =bac_counter '0'  then 
--		 -- if reset_n = '0'  then 
--            sin_counter <= (others => '0');
--        elsif lrck_lat = '1' and lrck = '0'  then            
--				if RAM(2) = "0000000000000001" then 
--					
--					sin_counter <= "0000000000000";	
--				--	RAM(2) := "0000000000000010";
--				elsif RAM(2) = "0000000000000000" then
--					sin_counter <= sin_counter + 1;
--					switch <= not switch ;
--					if sin_counter = endOfTone then 
--				--	RAM(2) <= "0000000000000000";
--				end if;
--          end if;
--        end if;
--      end if;
--    end process;
	 
	     process(clk)      
    begin
      if rising_edge(clk) then
        if reset_n = '0' then 

            bac_counter <= "00001";
        elsif lrck_lat = '1' and lrck = '0'  then  
          if bac_counter = "11111" then 
				
            bac_counter <= "00001";
          else  
            bac_counter <= bac_counter + 1;
						
          end if;
        end if;
      end if;
    end process;
	 
	 --reset the request signal
	   process (clk)
  begin
    if rising_edge(clk) then
      if reset_n = '0' then
        request <= '0';
      else
        if bac_counter = "11111" then
          request <= '1';
        elsif write = '1' and chipselect = '1' then
          request <= '0';  -- important to reset the irq
        end if;
      end if;
    end if;
  end process;
  
	 
    process(clk)
    begin
      if rising_edge(clk) then
        lrck_lat <= lrck;
      end if;
    end process;
	 



--v1: musicRom  port map (
--		clk ,
--		sin_counter (12 downto 0),
--		sin_out0);		
--v2: musicRom1  port map (
--		clk ,
--		sin_counter (12 downto 0),
--		sin_out1);

		
		
		---ending time  and  effect sound select
--	sin_out <= sin_out0;
	--	endOfTone <= "0100010011011" ;
endOfTone <= "01100010101010" when RAM(0) = "0000000000000001" or RAM(0) = "0000000000000010" else
				 "10100110011010" when RAM(0) = "0000000000000011" or RAM(0) = "0000000000000100" else
				 "11111100111000" when RAM(0) = "0000000000000101"or RAM(0) = "0000000000000110" else
				 "11111100111000";

--sin_out <= sin_out0 when RAM(0) = "0000000000000010" else
--		   sin_out1 when RAM(0) = "0000000000000001" else
--		   sin_out0;

		 --output address counter 
   process (clk)
  begin
    if rising_edge(clk) then
      if reset_n = '0' then
      	rom_addr<= (others => '0');        
      else 
       rom_addr <= std_logic_vector(sin_counter );
      end if;
    end if;
  end process;	



end architecture;


